import os
import shutil
from typing import Dict

import fasttext
import fasttext.util
import numpy as np
from fasttext.FastText import _FastText
from numpy import typing as npt
from sziszapangma.integration.service_core.embedding.embedding_base_processor \
    import EmbeddingBaseProcessor


class FasttextWebEmbeddingTransformer(EmbeddingBaseProcessor):
    _models = Dict[str, _FastText]

    def __init__(self):
        super().__init__()
        self._models = dict()

    def get_embedding(self, phrase: str, language: str) -> npt.NDArray[np.float64]:
        return self.get_model(language).get_word_vector(phrase)

    def get_model(self, language: str) -> _FastText:
        if language not in self._models:
            print(f'load model {language}')
            model_filename = f'cc.{language}.300.bin'
            model_path = f'/models/{model_filename}'
            print(f'{model_filename}  {model_path}')
            if not os.path.exists(model_path):
                full_model_name = fasttext.util.download_model(language,
                                                               if_exists='ignore')
                shutil.move(full_model_name, f'/models/{full_model_name}')
                print(f'downloaded {full_model_name}')
            self._models[language] = fasttext.load_model(model_path)
        return self._models[language]


if __name__ == '__main__':
    transformer = FasttextWebEmbeddingTransformer()
    transformer.get_model('pl')
    transformer.get_model('en')
    transformer.start_processor()
